import logging
import base64
from django.utils import timezone


def get_time_string():
    return timezone.make_naive(timezone.now()).strftime("%Y-%m-%d %H:%M:%S")


def get_request_headers(request):
    headers = {}
    for key, value in request.META.items():
        if key.startswith("HTTP_"):
            headers[key] = value
    return headers


def get_request_log_text(request):
    try:
        data = request.body
    except Exception:
        return "-"
    if isinstance(data, str):
        return data
    try:
        return data.decode("utf-8")
    except UnicodeDecodeError:
        try:
            return data.decode("gb18030")
        except UnicodeDecodeError:
            return "application/data;base64:" + "".join(
                base64.encodebytes(data).decode().splitlines()
            )


def get_response_log_data(response):
    try:
        data = request.content
    except Exception:
        return "-"
    if isinstance(data, str):
        return data
    try:
        return data.decode("utf-8")
    except UnicodeDecodeError:
        try:
            return data.decode("gb18030")
        except UnicodeDecodeError:
            return "application/data;base64:" + "".join(
                base64.encodebytes(data).decode().splitlines()
            )



from django.conf import settings

# 接口如果不指定methods，则默认同时支持get和post请求
DJANGO_APIS_ALLOWED_METHODS = getattr(
    settings,
    "DJANGO_APIS_ALLOWED_METHODS",
    ["GET", "POST"],
)
# 获取应用默认的apiview函数
# 一般应用需要自定义接口返回数据格式
DJANGO_APIS_APIVIEW = getattr(
    settings,
    "DJANGO_APIS_APIVIEW",
    None,
)
# 记录完整的请求和响应信息
DJANGO_APIS_ENABLE_REQUEST_LOG = getattr(
    settings,
    "DJANGO_APIS_ENABLE_REQUEST_LOG",
    False,
)
DJANGO_APIS_REQUEST_LOG_NAME = getattr(
    settings,
    "DJANGO_APIS_REQUEST_LOG_NAME",
    None,
)
# APIKEY设置
DJANGO_APIS_APIKEYS = getattr(
    settings,
    "DJANGO_APIS_APIKEYS",
    [],
)



from django.urls import path
from .views import ping
from .views import echo
from .views import getRequestInfo

urlpatterns = [
    path("ping", ping),
    path("echo", echo),
    path("getRequestInfo", getRequestInfo),
]



from django.test import TestCase
from django.test import LiveServerTestCase
from django.test import Client


class TestDjangoApisDebug(LiveServerTestCase):
    def test1(self):
        client = Client()
        response = client.get("/debug/ping")
        response_data = response.json()
        assert response_data["data"] == "pong"

    def test2(self):
        client = Client()
        response = client.get("/debug/echo")
        response_data = response.json()
        assert response_data["data"] is None
        assert response_data["success"] is False

    def test3(self):
        client = Client()
        response = client.post("/debug/echo")
        response_data = response.json()
        assert response_data["data"] is None
        assert response_data["success"] is False

    def test4(self):
        client = Client()
        response = client.post(
            "/debug/echo",
            data={
                "msg": "hello",
            },
            content_type="application/json",
        )
        response_data = response.json()
        assert response_data["data"] == "hello"
        assert response_data["success"]

    def test5(self):
        client = Client()
        response = client.post(
            "/debug/echo",
            data={
                "msg": "12345678901234567",
            },
            content_type="application/json",
        )
        response_data = response.json()
        assert response_data["data"] is None
        assert response_data["success"] is False

    def test6(self):
        client = Client()
        response = client.get("/debug/getRequestInfo")
        response_data = response.json()
        assert response_data["data"]
        assert response_data["success"]



from django_apis.views import get_apiview
from django_apis.views import get_json_payload
from django_apis.exceptions import InputValidationError
from .forms import EchoInput

apiview = get_apiview()


@apiview(methods="GET")
def ping(request):
    return "pong"


@apiview(methods="POST")
def echo(request):
    payload = get_json_payload(request)
    form = EchoInput(payload)
    if not form.is_valid():
        raise InputValidationError(form)
    return form.cleaned_data["msg"]


@apiview(methods="GET")
def getRequestInfo(request):
    result = {}
    for key, value in request.META.items():
        if isinstance(value, str):
            result[key] = value
    return result



from django.test import TestCase
from .response import SimpleJsonResponsePackage


class TestDjangoApis(TestCase):

    def test1(self):
        response = SimpleJsonResponsePackage("pong", None)
        data = response.pack()
        assert data
        assert data["data"] == "pong"

    def test2(self):
        response = SimpleJsonResponsePackage(None, RuntimeError(1, "info"))
        data = response.pack()
        assert data
        assert data["data"] is None
        assert data["success"] is False
        assert data["code"] == 1
        assert data["msg"] == "info"
